@@ -36,10 +36,11 @@
 #'  \itemize{
 #'    \item Kalliovirta L., Meitz M. and Saikkonen P. (2015) Gaussian Mixture Autoregressive model for univariate time series.
 #'          \emph{Journal of Time Series Analysis}, \strong{36}, 247-266.
-#'    \item References regarding the StMAR model will be updated after they are published.
+#'    \item References regarding the StMAR and G-StMAR models will be updated when they are published.
 #'  }
 
 isStationary_int <- function(p, M, params, restricted=FALSE) {
+  M = sum(M) # For G-StMAR models
   if(restricted==FALSE) {
     pars = matrix(params[1:(M*(p+2))], ncol=M)
     for(i1 in 1:M) {
@@ -57,9 +58,9 @@ isStationary_int <- function(p, M, params, restricted=FALSE) {
 }
 
 
-#' @title Check the stationary condition of specified GMAR or StMAR model.
+#' @title Check the stationary condition of specified GMAR, StMAR or G-StMAR model.
 #'
-#' @description \code{isStationary} checks the stationarity condition of the specified GMAR or StMAR model.
+#' @description \code{isStationary} checks the stationarity condition of the specified GMAR, StMAR or G-StMAR model.
 #'
 #' @inheritParams loglikelihood
 #' @details This function uses numerical approximations and it will falsely return \code{FALSE} for stationary models
@@ -74,6 +75,10 @@ isStationary_int <- function(p, M, params, restricted=FALSE) {
 #' params12t <- c(-0.3, 1, 0.9, 0.1, 0.8, 0.6, 0.7, 10, 12)
 #' isStationary(1, 2, params12t, StMAR=TRUE)
 #'
+#' # G-StMAR model
+#' params12gs <- c(1, 0.1, 1, 2, 0.2, 2, 0.8, 20)
+#' isStationary(1, c(1,1), params12gs, GStMAR=TRUE)
+#'
 #' # Restricted GMAR model
 #' params13r <- c(0.1, 0.2, 0.3, -0.99, 0.1, 0.2, 0.3, 0.5, 0.3)
 #' isStationary(1, 3, params13r, restricted=TRUE)
@@ -82,6 +87,10 @@ isStationary_int <- function(p, M, params, restricted=FALSE) {
 #' params22tr <- c(-0.1, -0.2, 0.01, 0.99, 0.3, 0.4, 0.9, 3, 13)
 #' isStationary(2, 2, params22tr, StMAR=TRUE, restricted=TRUE)
 #'
+#' # Restricted G-StMAR model
+#' params13gsr <- c(1, 2, 3, -0.99, 1, 2, 3, 0.5, 0.4, 20, 30)
+#' isStationary(1, c(1,2), params13gsr, GStMAR=TRUE, restricted=TRUE)
+#'
 #' # GMAR model as a mixture of AR(2) and AR(1) models
 #' R <- list(diag(1, ncol=2, nrow=2), as.matrix(c(1, 0)))
 #' params22c <- c(1.2, 0.8, 0.2, 0.3, 3.3, 0.7, 3, 0.8)
@@ -95,14 +104,15 @@ isStationary_int <- function(p, M, params, restricted=FALSE) {
 #'              constraints=TRUE, R=matrix(c(1, 0, 0, 0, 0, 1), ncol=2))
 #' @export
 
-isStationary <- function(p, M, params, StMAR=FALSE, restricted=FALSE, constraints=FALSE, R) {
-  checkPM(p, M)
-  if(length(params)!=nParams(p=p, M=M, StMAR=StMAR, restricted=restricted, constraints=constraints, R=R)) {
-    stop("the parameter vector is wrong dimension")
+isStationary <- function(p, M, params, StMAR=FALSE, GStMAR=FALSE, restricted=FALSE, constraints=FALSE, R) {
+  checkLogicals(StMAR=StMAR, GStMAR=GStMAR)
+  checkPM(p=p, M=M, GStMAR=GStMAR)
+  if(length(params)!=nParams(p=p, M=M, StMAR=StMAR, GStMAR=GStMAR, restricted=restricted, constraints=constraints, R=R)) {
+    stop("The parameter vector has wrong dimension")
   }
   if(constraints==TRUE) {
-    checkConstraintMat(p, M, R, restricted=restricted)
-    params = reformConstrainedPars(p, M, params, R=R, StMAR=StMAR, restricted=restricted)
+    checkConstraintMat(p=p, M=M, R, restricted=restricted)
+    params = reformConstrainedPars(p, M, params, R=R, StMAR=StMAR, GStMAR=GStMAR, restricted=restricted)
   }
   return(isStationary_int(p=p, M=M, params=params, restricted=restricted))
 }
@@ -110,7 +120,15 @@ isStationary <- function(p, M, params, StMAR=FALSE, restricted=FALSE, constraint
 
 #' @rdname isStationary_int
 
-isIdentifiable <- function(p, M, params, restricted=FALSE, StMAR=FALSE) {
+isIdentifiable <- function(p, M, params, restricted=FALSE, StMAR=FALSE, GStMAR=FALSE) {
+  if(GStMAR==TRUE) {
+    M1 = M[1]
+    M2 = M[2]
+    M = sum(M)
+    if(M1==1 & M2==1) {
+      return(TRUE)
+    }
+  }
   if(M==1) {
     return(TRUE)
   }
@@ -120,19 +138,49 @@ isIdentifiable <- function(p, M, params, restricted=FALSE, StMAR=FALSE) {
     alphas = c(alphas, 1-sum(alphas))
     if(StMAR==TRUE) {
       pars = rbind(pars, params[(M*(p+3)):(M*(p+4)-1)])
+    } else if(GStMAR==TRUE) {
+      pars1 = as.matrix(pars[,1:M1])
+      pars2 = as.matrix(pars[,(M1+1):M])
+      pars2 = rbind(pars2, params[(M*(p+3)):(M*(p+3)+M2-1)])
+      alphas1 = alphas[1:M1]
+      alphas2 = alphas[(M1+1):M]
     }
-  } else {
+  } else { # If restricted==TRUE
     pars = rbind(params[1:M], matrix(rep(params[(M+1):(M+p)], M), ncol=M), params[(M+p+1):(p+2*M)])
     alphas = params[(p+2*M+1):(3*M+p-1)]
     alphas = c(alphas, 1-sum(alphas))
     if(StMAR==TRUE) {
       pars = rbind(pars, params[(3*M+p):(4*M+p-1)])
+    } else if(GStMAR==TRUE) {
+      pars1 = as.matrix(pars[,1:M1])
+      pars2 = as.matrix(pars[,(M1+1):M])
+      pars2 = rbind(pars2, params[(3*M+p):(3*M+p+M2-1)])
+      alphas1 = alphas[1:M1]
+      alphas2 = alphas[(M1+1):M]
     }
   }
-  if(is.unsorted(rev(alphas), strictly=TRUE)) {
-    return(FALSE)
-  } else if(anyDuplicated(t(pars))!=0) {
-    return(FALSE)
+
+  if(GStMAR==TRUE) { # Check GMAR parameters and StMAR-parameters separately
+    if(M1>1) {
+      if(is.unsorted(rev(alphas1), strictly=TRUE)) {
+        return(FALSE)
+      } else if(anyDuplicated(t(pars1))!=0) {
+        return(FALSE)
+      }
+    }
+    if(M2>1) {
+      if(is.unsorted(rev(alphas2), strictly=TRUE)) {
+        return(FALSE)
+      } else if(anyDuplicated(t(pars2))!=0) {
+        return(FALSE)
+      }
+    }
+  } else { # if GStMAR is NOT true
+    if(is.unsorted(rev(alphas), strictly=TRUE)) {
+      return(FALSE)
+    } else if(anyDuplicated(t(pars))!=0) {
+      return(FALSE)
+    }
   }
   return(TRUE)
 }
@@ -154,9 +202,9 @@ checkAndCorrectData <- function(data, p) {
     data=t(data)   # The data matrix should contain observation per row (not per column)
   }
   if(ncol(data)>1) {
-    stop("the data has more than one columns")
+    stop("The data has more than one columns")
   } else if(anyNA(data)) {
-    stop("the data contains NA values, which is not supported")
+    stop("The data contains NA values, which is not supported")
   } else if(length(data)<p+2) {
     stop("The data must contain at least p+2 values")
   }
@@ -167,7 +215,7 @@ checkAndCorrectData <- function(data, p) {
 #' @title Check the parameter vector is specified correctly
 #'
 #' @description \code{parameterChecks} checks dimension, restrictions, stationarity and identifibility of the given parameters
-#'   of GMAR or StMAR model. Throws an error if any check fails.
+#'   of GMAR, StMAR or G-StMAR model. Throws an error if any check fails.
 #'
 #' @inheritParams loglikelihood
 #' @param params a real valued parameter vector specifying the model.
@@ -177,6 +225,8 @@ checkAndCorrectData <- function(data, p) {
 #'          \sigma_{m}^2)} and \strong{\eqn{\phi_{m}}}=\eqn{(\phi_{m,1},...,\phi_{m,p}), m=1,...,M}.}
 #'        \item{For \strong{StMAR} model:}{Size \eqn{(M(p+4)-1x1)} vector (\strong{\eqn{\theta, \nu}})\eqn{=}(\strong{\eqn{\upsilon_{1}}},...,\strong{\eqn{\upsilon_{M}}},
 #'          \eqn{\alpha_{1},...,\alpha_{M-1}, \nu_{1},...,\nu_{M}}).}
+#'        \item{For \strong{G-StMAR} model:}{Size \eqn{(M(p+3)+M2-1x1)} vector (\strong{\eqn{\theta, \nu}})\eqn{=}(\strong{\eqn{\upsilon_{1}}},...,\strong{\eqn{\upsilon_{M}}},
+#'          \eqn{\alpha_{1},...,\alpha_{M-1}, \nu_{M1+1},...,\nu_{M}}).}
 #'      }
 #' @param pars a parameter matrix containing parameters \eqn{(\phi_{i,0},...,\phi_{i,p}, \sigma_{i}^2)}
 #'  so that i:th column denotes i:th component.
@@ -184,28 +234,39 @@ checkAndCorrectData <- function(data, p) {
 #' @details Note that the "params" -parameter vector is assumed to be in the "standard" form for restricted models as well.
 #' @return Throws an error if any check fails. Doesn't return anything.
 
-parameterChecks <- function(p, M, params, pars, alphas, StMAR=FALSE, constraints=FALSE) {
+parameterChecks <- function(p, M, params, pars, alphas, StMAR=FALSE, GStMAR=FALSE, constraints=FALSE) {
   if(StMAR==TRUE) {
     dfs = params[(M*(p+3)):(M*(p+4)-1)]
     if(length(params)!=(M*(p+4)-1)) {
-      stop("the parameter vector is wrong dimension")
+      stop("The parameter vector has wrong dimension")
     } else if(any(dfs<=2)) {
-      stop("the degrees of freedom parameters have to be in the half open interval (2, 342-p]")
+      stop("The degrees of freedom parameters have to be larger than 2")
+    }
+  } else if(GStMAR==TRUE) {
+    M1 = M[1]
+    M2 = M[2]
+    M = sum(M)
+    dfs = params[(M*(p+3)):(M*(p+3)+M2-1)]
+    if(length(params)!=M*(p+3)-1+M2) {
+      stop("The parameter vector has wrong dimension")
+    } else if(any(dfs<=2)) {
+      stop("The degrees of freedom parameters have to be larger than 2")
     }
   } else {
     if(length(params)!=(M*(p+3)-1)) {
-      stop("the parameter vector is wrong dimension")
+      stop("The parameter vector has wrong dimension")
     }
   }
   if(M>=2 & sum(alphas[-M])>=1) {
-    stop("the mixing weights don't sum to one")
+    stop("The mixing weights don't sum to one")
   } else if(any(pars[p+2,]<=0)) {
-    stop("component variances has to be larger than zero")
+    stop("Component variances have to be larger than zero")
   } else if(!isStationary_int(p, M, params, restricted=FALSE)) {
-    stop("the model doesn't satisfy the stationary condition")
+    stop("The model doesn't satisfy the stationary condition")
   }
-  if(constraints==FALSE & !isIdentifiable(p, M, params, restricted=FALSE, StMAR=StMAR)) {
-    stop("the model doesn't satisfy the identification conditions")
+  if(GStMAR==TRUE) { M = c(M1, M2)}
+  if(constraints==FALSE & !isIdentifiable(p, M, params, restricted=FALSE, StMAR=StMAR, GStMAR=GStMAR)) {
+    stop("The model doesn't satisfy the identification conditions")
   }
 }
 
@@ -217,34 +278,35 @@ parameterChecks <- function(p, M, params, pars, alphas, StMAR=FALSE, constraints
 #' @return Doesn't return anything, but throws an error if finds out that something is wrong.
 
 checkConstraintMat <- function(p, M, R, restricted=FALSE) {
+  M = sum(M) # For G-StMAR
   if(restricted==TRUE) {
     if(missing(R)) {
-      stop("the constraint matrix R needs to be provided")
+      stop("The constraint matrix R needs to be provided")
     } else if(!is.matrix(R)) {
-      stop("the constraint matrix R has to be a matrix")
+      stop("The constraint matrix R has to be a matrix")
     } else if(nrow(as.matrix(R))!=p) {
-      stop("the constraint matrix R is wrong dimension")
+      stop("The constraint matrix R has wrong dimension")
     } else if(ncol(as.matrix(R))>p) {
       stop("Why would you need a constraint matrix with more columns than rows? Please make sure your constraints make sense!")
     } else if(qr(as.matrix(R))$rank!=ncol(as.matrix(R))) {
-      stop("the constraint matrix is not full column rank")
+      stop("The constraint matrix is not full column rank")
     }
   } else {
     if(missing(R)) {
       stop("a list of constraint matrices R needs to be provided")
     }
     if(!is.list(R) | length(R)!=M) {
-      stop("the argument R should be a list of M constraint matrices R_{m} - one for each component model")
+      stop("The argument R should be a list of M constraint matrices R_{m} - one for each component model")
     }
     for(i1 in 1:M) {
       R0 = as.matrix(R[[i1]])
       q = ncol(R0)
       if(nrow(R0)!=p) {
-        stop(paste("the constraint matrix R for regime", i1 ,"is wrong dimension"))
+        stop(paste("The constraint matrix R for regime", i1 ,"has wrong dimension"))
       } else if(q>p) {
         stop("Why would you need a constraint matrix with more columns than rows? Please make sure your constraints make sense!")
       } else if(qr(R0)$rank!=ncol(R0)) {
-        stop(paste("the constraint matrix R for regime", i1 ,"is not full column rank"))
+        stop(paste("The constraint matrix R for regime", i1 ,"is not full column rank"))
       }
     }
   }
@@ -256,53 +318,89 @@ checkConstraintMat <- function(p, M, R, restricted=FALSE) {
 #' @inheritParams loglikelihood_int
 #' @return Doesn't return anything, but throws and error if something is wrong.
 
-checkPM <- function(p, M) {
-  if(p<1) {
-    stop("argument p has to be equal or larger than one")
-  } else if(M<1) {
-    stop("argument M has to be equal or larger than one")
-  } else if(p%%1!=0) {
-    stop("argument p has to be positive integer")
-  } else if(M%%1!=0) {
-    stop("argument M has to be positive integer")
+checkPM <- function(p, M, GStMAR=FALSE) {
+  if(GStMAR==TRUE) {
+    if(length(M)!=2) {
+      stop("For G-StMAR model the argument M should be a vector of length 2")
+    }
+    for(i1 in 1:2) {
+      if(M[i1]<1 | M[i1]%%1!=0) {
+        stop(paste0("Argument M[", i1, "] has to be positive integer"))
+      }
+    }
+  } else if(M<1 | M%%1!=0) {
+      stop("Argument M has to be positive integer")
+  }
+  if(p<1 | p%%1!=0) {
+    stop("Argument p has to be positive integer")
   }
 }
 
 #' @title Calculate the number of parameters
 #'
-#' @description \code{nParams} calculates the number of parameters
+#' @description \code{nParams} calculates the number of parameters that should be in the parameter vector.
 #' @inheritParams loglikelihood_int
 #' @return returns the number of parameters.
 
-nParams <- function(p, M, StMAR=FALSE, restricted=FALSE, constraints=FALSE, R) {
+nParams <- function(p, M, StMAR=FALSE, GStMAR=FALSE, restricted=FALSE, constraints=FALSE, R) {
   if(restricted==FALSE) {
-    if(StMAR==FALSE) {
-      if(constraints==FALSE) {
-        d = M*(p+3)-1
-      } else {
-        d = 3*M-1+sum(vapply(1:M, function(i1) ncol(as.matrix(R[[i1]])), numeric(1)))
-      }
-    } else {
+    if(StMAR==TRUE) {
       if(constraints==FALSE) {
         d = M*(p+4)-1
       } else {
         d = 4*M-1+sum(vapply(1:M, function(i1) ncol(as.matrix(R[[i1]])), numeric(1)))
       }
-    }
-  } else {
-    if(StMAR==FALSE) {
+    } else if(GStMAR==TRUE) {
+      M1 = M[1]
+      M2 = M[2]
+      M = sum(M)
       if(constraints==FALSE) {
-        d = 3*M+p-1
+        d = M*(p+3)+M2-1
       } else {
-        d = 3*M+ncol(as.matrix(R))-1
+        d = 3*M+M2-1+sum(vapply(1:M, function(i1) ncol(as.matrix(R[[i1]])), numeric(1)))
       }
-    } else {
+    } else { # If GMAR==TRUE
+      if(constraints==FALSE) {
+        d = M*(p+3)-1
+      } else {
+        d = 3*M-1+sum(vapply(1:M, function(i1) ncol(as.matrix(R[[i1]])), numeric(1)))
+      }
+    }
+  } else { # if restricted==TRUE
+    if(StMAR==TRUE) {
       if(constraints==FALSE) {
         d = 4*M+p-1
       } else {
         d = 4*M+ncol(as.matrix(R))-1
       }
+    } else if(GStMAR==TRUE) {
+      M1 = M[1]
+      M2 = M[2]
+      M = sum(M)
+      if(constraints==FALSE) {
+        d = 3*M+M2+p-1
+      } else {
+        d = 3*M+M2+ncol(as.matrix(R))-1
+      }
+    } else { # if GMAR=TRUE
+      if(constraints==FALSE) {
+        d = 3*M+p-1
+      } else {
+        d = 3*M+ncol(as.matrix(R))-1
+      }
     }
   }
   return(d)
 }
+
+
+#' @title Check that the logical arguments StMAR and GStMAR don't conflict
+#'
+#' @description \code{checkLogicals} checks that the logical arguments StMAR and GStMAR don't conflict.
+#' @inheritParams loglikelihood_int
+#' @return Doesn't return anything, but throws and error if something is wrong.
+checkLogicals <- function(StMAR, GStMAR) {
+  if(StMAR==TRUE & GStMAR==TRUE) {
+    stop("Arguments StMAR and GStMAR are both set to be TRUE. You obviously can't consider both models at the same time!")
+  }
+}
